(this.webpackJsonpsqllineagejs=this.webpackJsonpsqllineagejs||[]).push([[0],{131:function(e,t,r){},316:function(e,t,r){"use strict";r.r(t);var n=r(11),o=r(16),i=r.n(o),a=r(65),s=r.n(a),c=(r(131),r(120)),d=r(121),l=r(125),h=r(124),u=r(122),b=r.n(u),g=r(123),j=r.n(g),O=r(66);r.n(O).a.use(j.a);var p=function(e){Object(l.a)(r,e);var t=Object(h.a)(r);function r(e){var n;return Object(c.a)(this,r),(n=t.call(this,e)).state={error:null,isLoaded:!1,elements:[]},n}return Object(d.a)(r,[{key:"componentDidMount",value:function(){var e=this,t=new URL(window.location.href),r=Object({NODE_ENV:"production",PUBLIC_URL:"",WDS_SOCKET_HOST:void 0,WDS_SOCKET_PATH:void 0,WDS_SOCKET_PORT:void 0,FAST_REFRESH:!0}).REACT_APP_BACKEND_PORT,n=r?t.origin.replace(t.port,r):t.origin;fetch("".concat(n,"/lineage"),{method:"POST",body:JSON.stringify(Object.fromEntries(t.searchParams)),headers:new Headers({"Content-Type":"application/json"})}).then((function(e){return e.json()})).then((function(t){e.setState({isLoaded:!0,elements:t})}),(function(t){e.setState({isLoaded:!0,error:t})}))}},{key:"render",value:function(){var e=this.state,t=e.error,r=e.isLoaded,o=e.elements;if(t)return Object(n.jsxs)("div",{children:["Error: ",t.message]});if(r){return Object(n.jsx)(b.a,{elements:o,stylesheet:[{selector:"node",style:{height:10,width:10,content:"data(id)","text-valign":"top","text-halign":"right","font-size":10,color:"#35393e","background-color":"#3499d9","border-color":"#000","border-width":1,"border-opacity":.8}},{selector:"edge",style:{width:1,"line-color":"#9ab5c7","target-arrow-color":"#9ab5c7","target-arrow-shape":"triangle","arrow-scale":.8,"curve-style":"bezier"}}],style:{width:"1920px",height:"1080px"},layout:{name:"dagre",rankDir:"LR",rankSep:200},zoom:1,minZoom:.5,maxZoom:2,wheelSensitivity:.2})}return Object(n.jsx)("div",{children:"Loading..."})}}]),r}(i.a.Component);s.a.render(Object(n.jsx)(i.a.StrictMode,{children:Object(n.jsx)(p,{})}),document.getElementById("root"))}},[[316,1,2]]]);
//# sourceMappingURL=main.456d1fcf.chunk.js.map